@@ -1,4 +1,5 @@
 import argparse
+import os
 import pandas as pd
 import numpy as np
 from dexom_python.imat_functions import imat
@@ -9,17 +10,19 @@ from warnings import warn
 
 
 class RxnEnumSolution(object):
-    def __init__(self,
-                 all_solutions, unique_solutions, all_binary, unique_binary, all_reactions=None, unique_reactions=None):
+    def __init__(self, all_solutions, unique_solutions, all_binary, unique_binary,
+                 all_reactions=None, unique_reactions=None, objective_value=-1.):
         self.all_solutions = all_solutions
         self.unique_solutions = unique_solutions
         self.all_binary = all_binary
         self.unique_binary = unique_binary
         self.all_reactions = all_reactions
         self.unique_reactions = unique_reactions
+        self.objective_value = objective_value
 
 
-def rxn_enum(model, reaction_weights, prev_sol, rxn_list=[], eps=1., thr=1e-1, obj_tol=1e-2):
+def rxn_enum(model, reaction_weights, prev_sol=None, rxn_list=[], eps=1e-4, thr=1e-4, obj_tol=1e-3, out_path='enum_rxn',
+             save=False):
     """
     Reaction enumeration method
 
@@ -40,10 +43,18 @@ def rxn_enum(model, reaction_weights, prev_sol, rxn_list=[], eps=1., thr=1e-1, o
         tolerance for imat
     obj_tol: float
         variance allowed in the objective_values of the solutions
+    out_path: str
+        path to which the solutions are saved if save==True
+    save: bool
+        if True, every individual solution is saved in the iMAT solution format
     Returns
     -------
     solution: RxnEnumSolution object
     """
+    if prev_sol is None:
+        prev_sol = imat(model, reaction_weights, epsilon=eps, threshold=thr, full=False)
+    else:
+        model = create_enum_variables(model=model, reaction_weights=reaction_weights, eps=eps, thr=thr, full=False)
     tol = model.solver.configuration.tolerances.feasibility
     prev_sol_bin = (np.abs(prev_sol.fluxes) >= thr-tol).values.astype(int)
     optimal_objective_value = prev_sol.objective_value - prev_sol.objective_value * obj_tol
@@ -54,7 +65,8 @@ def rxn_enum(model, reaction_weights, prev_sol, rxn_list=[], eps=1., thr=1e-1, o
     unique_solutions_binary = [prev_sol_bin]
     all_reactions = []  # for each solution, save which reaction was activated/inactived by the algorithm
     unique_reactions = []
-
+    if save:  # when saving each individual solution, ensure that the out_path is a directory
+        os.makedirs(out_path, exist_ok=True)
     if not rxn_list:
         rxns = list(model.reactions)
         rxn_list = [r.id for r in rxns]
@@ -85,6 +97,9 @@ def rxn_enum(model, reaction_weights, prev_sol, rxn_list=[], eps=1., thr=1e-1, o
                                         unique_solutions.append(temp_sol)
                                         unique_solutions_binary.append(temp_sol_bin)
                                         unique_reactions.append(rid+'_backwards')
+                                        if save:
+                                            filename = out_path+'_solution_'+str(len(unique_solutions)-1)+'.csv'
+                                            write_solution(model, temp_sol, thr, filename)
                             except:
                                 print('An error occurred with reaction %s_reverse. '
                                       'Check feasibility of the model when this reaction is irreversible.' % rid)
@@ -107,6 +122,9 @@ def rxn_enum(model, reaction_weights, prev_sol, rxn_list=[], eps=1., thr=1e-1, o
                                 unique_solutions.append(temp_sol)
                                 unique_solutions_binary.append(temp_sol_bin)
                                 unique_reactions.append(rid)
+                                if save:
+                                    filename = out_path+'_solution_'+str(len(unique_solutions)-1)+'.csv'
+                                    write_solution(model, temp_sol, thr, filename)
                     except:
                         if prev_sol_bin[idx] == 1:
                             print('An error occurred with reaction %s. '
@@ -115,7 +133,7 @@ def rxn_enum(model, reaction_weights, prev_sol, rxn_list=[], eps=1., thr=1e-1, o
                             print('An error occurred with reaction %s. '
                                   'Check feasibility of the model when this reaction is irreversible' % rid)
     solution = RxnEnumSolution(all_solutions, unique_solutions, all_solutions_binary, unique_solutions_binary,
-                               all_reactions, unique_reactions)
+                               all_reactions, unique_reactions, prev_sol.objective_value)
     return solution
 
 
@@ -135,7 +153,7 @@ if __name__ == '__main__':
                         help='Activation threshold for highly expressed reactions')
     parser.add_argument('--threshold', type=float, default=1e-5, help='Activation threshold for all reactions')
     parser.add_argument('-t', '--timelimit', type=int, default=None, help='Solver time limit')
-    parser.add_argument('--tol', type=float, default=1e-6, help='Solver feasibility tolerance')
+    parser.add_argument('--tol', type=float, default=1e-7, help='Solver feasibility tolerance')
     parser.add_argument('--mipgap', type=float, default=1e-3, help='Solver MIP gap tolerance')
     parser.add_argument('--obj_tol', type=float, default=1e-3,
                         help='objective value tolerance, as a fraction of the original value')
@@ -176,11 +194,7 @@ if __name__ == '__main__':
         initial_solution = imat(model, reaction_weights, epsilon=args.epsilon, threshold=args.threshold)
 
     solution = rxn_enum(model=model, rxn_list=rxn_list, prev_sol=initial_solution, reaction_weights=reaction_weights,
-                        eps=args.epsilon, thr=args.threshold, obj_tol=args.obj_tol)
+                        eps=args.epsilon, thr=args.threshold, obj_tol=args.obj_tol, out_path=args.output, save=args.save)
 
     uniques = pd.DataFrame(solution.unique_binary)
     uniques.to_csv(args.output+'_solutions.csv')
-
-    if args.save:
-        for i in range(1, len(solution.unique_solutions)):
-            write_solution(model, solution.unique_solutions[i], args.threshold, args.output+'_solution_'+str(i)+'.csv')
